#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MIT License
#
# Copyright (c) 2020 Louis Richard
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so.

import string


def make_labels(axs, pos, pad=0):
    """Add subplots labels to axes

    Parameters
    ----------
    axs : numpy.ndarray
        Array of subplots axes.

    pos : list or numpy.ndarray
        Position of the text in the axis.

    pad : int
        (Option) Offset in axis counter.

    Returns
    -------
    axs : numpy.ndarray
        Array of subplots axes with labels.

    """

    lbl = string.ascii_lowercase[pad:len(axs) + pad]

    for label, axis in zip(lbl, axs):
        axis.text(pos[0], pos[1], "({})".format(label), transform=axis.transAxes)

    return axs



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MIT License
#
# Copyright (c) 2020 Louis Richard
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so.

import numpy as np
import xarray as xr

from . import resample, dot, normalize, cross


def eb_nrf(e, b, v, flag=0):
    """Find E and B in MP system given B and MP normal vector.

    Parameters
    ----------
    e : xarray.DataArray
        Time series of the electric field.

    b : xarray.DataArray
        Time series of the magnetic field.

    v : xarray.DataArray
        Normal vector.

    flag : int or ndarray
        to fill.

    Returns
    -------
    out : xarray.DataArray
        to fill.

    """

    if isinstance(flag, int):
        if flag == 1:
            flag_case = "b"
        else:
            flag_case = "a"
        
        l_direction = None

    elif isinstance(flag, np.ndarray) and np.size(flag) == 3:
        l_direction = flag
        flag_case = "c"
    else:
        raise TypeError("Invalid flag type")

    if flag_case == "a":
        be = resample(b, e).data

        nl = be / np.linalg.norm(be, axis=0)[:, None]  # along the B
        nn = np.cross(np.cross(be, v), be)  # closest to given vn vector
        nn = nn / np.linalg.norm(nn)[:, None]
        nm = np.cross(nn, nl)  # in (vn x b) direction

        # estimate e in new coordinates
        en = dot(e, nn)
        el = dot(e, nl)
        em = dot(e, nm)
        emp = np.hstack([el, em, en])

    elif flag_case == "b":
        nn = v / np.linalg.norm(v)
        nm = normalize(cross(nn, b.mean(dim="time")))
        nl = cross(nm, nn)

        # estimate e in new coordinates
        en = dot(e, nn)
        el = dot(e, nl)
        em = dot(e, nm)
        emp = np.hstack([el, em, en])

    elif flag_case == "c":
        nn = normalize(v)
        nm = normalize(np.cross(nn, l_direction))
        nl = cross(nm, nn)

        # estimate e in new coordinates
        en = dot(e, nn)
        el = dot(e, nl)
        em = dot(e, nm)

        emp = np.hstack([el, em, en])
    else:
        raise ValueError("Invalid flag_case")

    out = xr.DataArray(e.time.data, emp, e.attrs)

    return out



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MIT License
#
# Copyright (c) 2020 Louis Richard
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so.

import numpy as np
import xarray as xr


def struct_func(x, scale, order):
    """Returns the structure function of a time series

    .. math::

       y= \\frac{1}{N-s}\\sum_{i=1}^{N-s}(x_i - x_{i+s})^o

    where :math:`s` is the scale, and :math:`o` is the order.

    Parameters
    ----------
    x : xarray.DataArray
        Input time series.

    scale : list or ndarray
        A list or an array containing the scales to calculate.

    order : int
        Order of the exponential of the structure function.

    Returns
    -------
    values : xarray.DataArray
        An xarray containing the structure functions, one per product in the original time
        series. The index coordinate contains the scale value, and the attribute 'order' keeps a
        record on the order used for its calculation.

    """

    if scale is None:
        scale = [1]

    data = x.data

    if len(data.shape) == 1:
        data = data[:, np.newaxis]
    else:
        pass

    result = []
    for s in scale:
        result.append(np.mean(np.abs((data[s:, :] - data[:-s, :]) ** order), axis=0))

    result = np.array(result)

    c = x.dims[1]
    cols = x.coords[c].data

    result = xr.DataArray(result, coords=[scale, cols], dims=["scale", c], attrs=x.attrs)

    result.attrs['order'] = order

    return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MIT License
#
# Copyright (c) 2020 Louis Richard
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so.

import numpy as np
import xarray as xr

from ..pyrf import calc_dt


def psd_rebin(vdf, phi, energy0, energy1, step_table):
    """Convert burst mode distribution into 64 energy channel distribution.
    Functions takes the burst mode distribution sampled in two energy tables and converts to a
    single energy table with 64 energy channels. Time resolution is halved and phi angles are
    averaged over adjacent times.

    Parameters
    ----------
    vdf : xarray.Dataset
        Time series of the particle distribution.

    phi : xarray.DataArray
        Time series of the phi angles.

    energy0 : xarray.DataArray or numpy.ndarray
        Energy table 0.

    energy1 : xarray.DataArray or numpy.ndarray
        Energy table 1.

    step_table : xarray.DataArray
        Time series of the stepping table between energies (burst).

    Returns
    -------
    time_r : ndarray
        Revised time steps.

    vdf_r : ndarray
        Rebinned particle distribution.

    energy_r : ndarray
        Revised energy table.

    phi_r : ndarray
        Time series of the recalculated phi angle.

    Notes
    -----
    I'm assuming no gaps in the burst data interval. If there is a gap use time_clip before running.
    To be updated later.
    
    """

    if isinstance(energy0, xr.DataArray):
        energy0 = energy0.data
    else:
        pass

    if isinstance(energy1, xr.DataArray):
        energy1 = energy1.data
    else:
        pass

    step_table = step_table.data

    # Sort energy levels
    energy_r = np.sort(np.hstack([energy0, energy1]))

    # Define new times
    dt = calc_dt(vdf.data)
    time_r = vdf.time.data[:-1:2] + int(dt * 1e9 / 2)

    vdf_r, phi_r = [np.zeros((len(time_r), 64, 32, 16)), np.zeros((len(time_r), 32))]

    phi_s = np.roll(phi.data, 2, axis=1)
    phi_s[:, 0] = phi_s[:, 0] - 360

    time_indices = np.arange(0, len(vdf.time) - 1, 2)

    for new_el_num, ii in enumerate(time_indices):
        if phi.data[ii, 0] > phi.data[ii + 1, 0]:
            phi_r[new_el_num, :] = (phi.data[ii, :] + phi_s[ii + 1, :]) / 2

            vdf_temp = np.roll(np.squeeze(vdf.data.data[ii + 1, ...]), 2, axis=1)

            if step_table[ii]:
                vdf_r[new_el_num, 1:64:2, ...] = vdf.data.data[ii, ...]
                vdf_r[new_el_num, 0:63:2, ...] = vdf_temp
            else:
                vdf_r[new_el_num, 1:64:2, ...] = vdf.data.data[ii, ...]
                vdf_r[new_el_num, 0:63:2, ...] = vdf_temp

        else:
            phi_r[new_el_num, :] = (phi.data[ii, :] + phi.data[ii + 1, :]) / 2

            if step_table[ii]:
                vdf_r[new_el_num, 1:64:2, ...] = vdf.data.data[ii, ...]
                vdf_r[new_el_num, 0:63:2, ...] = vdf.data.data[ii + 1, ...]
            else:
                vdf_r[new_el_num, 1:64:2, ...] = vdf.data.data[ii + 1, ...]
                vdf_r[new_el_num, 0:63:2, ...] = vdf.data.data[ii, ...]

    return time_r, vdf_r, energy_r, phi_r



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MIT License
#
# Copyright (c) 2020 Louis Richard
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so.

import os

from .mms_config import CONFIG
from .list_files import list_files


def copy_files(var, tint, mms_id, target_dir="./data/"):
    """Copy files from NAS24 to the target path

    Parameters
    ----------
    var : dict
        Dictionary containing 4 keys
            * var["inst"] : name of the instrument
            * var["tmmode"] : data rate
            * var["lev"] : data level
            * var["dtype"] : data type

    tint : list of str
        Time interval

    mms_id : str or int
        Index of the spacecraft

    target_dir : str, optional
        Target path. Default is './data/'

    """

    mms_path = CONFIG["local_data_dir"] + "/"

    files = list_files(tint, mms_id, var)

    for file in files:
        relative_path = os.path.split(file)[0].replace(mms_path, "")
        path = os.path.join(target_dir, relative_path)

        if not os.path.exists(path):
            os.makedirs(path)
            os.popen('cp {} {}'.format(file, os.path.join(path, os.path.split(file)[1])))
        else:
            os.popen('cp {} {}'.format(file, os.path.join(path, os.path.split(file)[1])))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MIT License
#
# Copyright (c) 2020 Louis Richard
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so.


import numpy as np
import xarray as xr

from cdflib import CDF, cdfepoch
from dateutil import parser as date_parser

from ..pyrf import datetime_to_tt2000


def get_ts(file_path, cdf_name, tint):
    """
    Read field named cdf_name in file and convert to time series.

    Parameters
    ----------
    file_path : str
        Path of the cdf file.

    cdf_name : str
        Name of the target variable in the cdf file.

    tint : list of str
        Time interval.

    Returns
    -------
    out : xarray.DataArray
        Time series of the target variable in the selected time interval.

    """

    # Convert time interval to epochs
    tint = list(map(date_parser.parse, tint))
    tint = list(map(datetime_to_tt2000, tint))
    tint = list(map(cdfepoch.parse, tint))

    x, y, z, w = [{}, {}, {}, {}]
    out_dict = {}

    with CDF(file_path) as f:
        depend0_key = f.varattsget(cdf_name)["DEPEND_0"]

        x["data"] = f.varget(depend0_key, starttime=tint[0], endtime=tint[1])

        if f.varinq(depend0_key)["Data_Type_Description"] == "CDF_TIME_TT2000":
            x["data"] = cdfepoch.to_datetime(x["data"], to_np=True)

        x["atts"] = {}

        for k in f.varattsget(depend0_key):
            x["atts"][k] = f.varattsget(depend0_key)[k]
            if isinstance(x["atts"][k], str) and x["atts"][k] in f.cdf_info()["zVariables"] \
                    and not k == "LABLAXIS":
                try:
                    # If array
                    x["atts"][k] = f.varget(x["atts"][k], starttime=tint[0], endtime=tint[1])
                except IndexError:
                    # If scalar
                    x["atts"][k] = f.varget(x["atts"][k])

        if "DEPEND_1" in f.varattsget(cdf_name) or "REPRESENTATION_1" in f.varattsget(cdf_name):
            try:
                depend1_key = f.varattsget(cdf_name)["DEPEND_1"]
            except KeyError:
                depend1_key = f.varattsget(cdf_name)["REPRESENTATION_1"]

            if depend1_key == "x,y,z":
                y["data"], y["atts"] = [np.array(depend1_key.split(",")), {"LABLAXIS": "comp"}]
            else:
                try:
                    y["data"] = f.varget(depend1_key, starttime=tint[0], endtime=tint[1])
                except IndexError:
                    y["data"] = f.varget(depend1_key)

                if len(y["data"]) == 1:
                    y["data"] = y["data"][0]

                # If vector components remove magnitude index

                if len(y["data"]) == 4 and all(y["data"] == ["x", "y", "z", "r"]):
                    y["data"] = y["data"][:-1]
                # if y is 2d get only first row assuming that the bins are the same
                elif y["data"].ndim == 2:
                    try:
                        y["data"] = y["data"][0, :]
                    except IndexError:
                        pass

                y["atts"] = {}

                # Get attributes
                for k in f.varattsget(depend1_key):
                    y["atts"][k] = f.varattsget(depend1_key)[k]

                    if isinstance(y["atts"][k], str) and y["atts"][k] in f.cdf_info()["zVariables"]:
                        if k not in ["DEPEND_0", "LABLAXIS"]:
                            try:
                                y["atts"][k] = f.varget(y["atts"][k], starttime=tint[0],
                                                        endtime=tint[1])
                            except IndexError:
                                y["atts"][k] = f.varget(y["atts"][k])
                            # If atts is 2D get only first row
                            if y["atts"][k].ndim == 2:
                                try:
                                    y["atts"][k] = y["atts"][k][0, :]
                                except IndexError:
                                    pass

                # Remove spaces in label
                try:
                    y["atts"]["LABLAXIS"] = y["atts"]["LABLAXIS"].replace(" ", "_")

                    if y["atts"]["LABLAXIS"] == "Diffential_energy_channels":
                        y["atts"]["LABLAXIS"] = "Differential_energy_channels"

                except KeyError:
                    y["atts"]["LABLAXIS"] = "comp"

        elif "afg" in cdf_name or "dfg" in cdf_name:
            y["data"] = ["x", "y", "z"]
            y["atts"] = {"LABLAXIS": "comp"}

        if "DEPEND_2" in f.varattsget(cdf_name) or "REPRESENTATION_2" in f.varattsget(cdf_name):
            try:
                depend2_key = f.varattsget(cdf_name)["DEPEND_2"]
            except KeyError:
                depend2_key = f.varattsget(cdf_name)["REPRESENTATION_2"]

            if depend2_key == "x,y,z":
                z["data"] = np.array(depend2_key.split(","))

                z["atts"] = {"LABLAXIS": "comp"}
            else:
                z["data"] = f.varget(depend2_key)

                if len(z["data"]) == 1:
                    z["data"] = z["data"][0]

                z["atts"] = {}

                for k in f.varattsget(depend2_key):
                    z["atts"][k] = f.varattsget(depend2_key)[k]

                    if isinstance(z["atts"][k], str) and z["atts"][k] in f.cdf_info()[
                        "zVariables"] and \
                            not k == "DEPEND_0":
                        z["atts"][k] = f.varget(z["atts"][k], starttime=tint[0], endtime=tint[1])

                if "LABLAXIS" not in z["atts"].keys():
                    z["atts"]["LABLAXIS"] = "comp"

        if "DEPEND_3" in f.varattsget(cdf_name) or "REPRESENTATION_3" in f.varattsget(cdf_name) \
                and f.varattsget(cdf_name)["REPRESENTATION_3"] != "x,y,z":

            try:
                depend3_key = f.varattsget(cdf_name)["DEPEND_3"]
            except KeyError:
                depend3_key = f.varattsget(cdf_name)["REPRESENTATION_3"]

            w["data"] = f.varget(depend3_key)

            if len(w["data"]) == 1:
                w["data"] = w["data"][0]

            if w["data"].ndim == 2:
                try:
                    w["data"] = w["data"][0, :]
                except IndexError:
                    pass

            w["atts"] = {}
            for k in f.varattsget(depend3_key):
                w["atts"][k] = f.varattsget(depend3_key)[k]

                if isinstance(w["atts"][k], str) and w["atts"][k] in f.cdf_info()["zVariables"] \
                        and not k == "DEPEND_0":
                    w["atts"][k] = f.varget(w["atts"][k], starttime=tint[0], endtime=tint[1])

            if "LABLAXIS" not in w["atts"]:
                w["atts"]["LABLAXIS"] = "comp"

        if "sector_mask" in cdf_name:
            depend1_key = f.varattsget(cdf_name.replace("sector_mask", "intensity"))["DEPEND_1"]

            y["data"] = f.varget(depend1_key)
            y["atts"] = f.varattsget(depend1_key)

            y["atts"]["LABLAXIS"] = y["atts"]["LABLAXIS"].replace(" ", "_")

        if "edp_dce_sensor" in cdf_name:
            y["data"] = ["x", "y", "z"]
            y["atts"] = {"LABLAXIS": "comp"}

        out_dict["data"] = f.varget(cdf_name, starttime=tint[0], endtime=tint[1])

        if out_dict["data"].ndim == 2 and out_dict["data"].shape[1] == 4:
            out_dict["data"] = out_dict["data"][:, :-1]

        out_dict["atts"] = f.varattsget(cdf_name)

    if x and not y and not z and not w:
        dims = ["time"]
        coords_data = [x["data"]]
        coords_atts = [x["atts"]]

    elif x and y and not z and not w:
        dims = ["time", y["atts"]["LABLAXIS"]]
        coords_data = [x["data"], y["data"]]
        coords_atts = [x["atts"], y["atts"]]

    elif x and y and z and not w:
        if y["atts"]["LABLAXIS"] == z["atts"]["LABLAXIS"]:
            y["atts"]["LABLAXIS"] = "rcomp"
            z["atts"]["LABLAXIS"] = "ccomp"

        dims = ["time", y["atts"]["LABLAXIS"], z["atts"]["LABLAXIS"]]
        coords_data = [x["data"], y["data"], z["data"]]
        coords_atts = [x["atts"], y["atts"], z["atts"]]

    elif x and y and z and w:
        if z["atts"]["LABLAXIS"] == w["atts"]["LABLAXIS"]:
            z["atts"]["LABLAXIS"] = "rcomp"
            w["atts"]["LABLAXIS"] = "ccomp"

        dims = ["time", y["atts"]["LABLAXIS"], z["atts"]["LABLAXIS"], w["atts"]["LABLAXIS"]]
        coords_data = [x["data"], y["data"], z["data"], w["data"]]
        coords_atts = [x["atts"], y["atts"], z["atts"], w["atts"]]

    else:
        raise NotImplementedError

    out = xr.DataArray(out_dict["data"], coords=coords_data, dims=dims, attrs=out_dict["atts"])

    for dim, coord_atts in zip(dims, coords_atts):
        out[dim].attrs = coord_atts

    return out



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MIT License
#
# Copyright (c) 2020 Louis Richard
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so.

import numpy as np

from .cross import cross
from .dot import dot


def c_4_k(r_list):
    """Calculates reciprocal vectors in barycentric coordinates.

    Parameters
    ----------
    r_list : list of xarray.DataArray
        Position of the spacecrafts.

    Returns
    -------
    k_list : list of xarray.DataArray
        Reciprocal vectors in barycentric coordinates.

    Notes
    -----
    The units of reciprocal vectors are the same as [1/r].

    """

    mms_list = np.arange(4)

    k_list = [r_list[0].copy()] * 4

    for i, j, k, l in zip(mms_list, np.roll(mms_list, 1), np.roll(mms_list, 2),
                          np.roll(mms_list, 3)):
        cc = cross(r_list[k]-r_list[j], r_list[l]-r_list[j])

        dr12 = r_list[i]-r_list[j]

        k_list[i] = cc/dot(cc, dr12)

    return k_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MIT License
#
# Copyright (c) 2020 Louis Richard
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so.


CONFIG = {"local_data_dir": "/Volumes/mms",
          'mirror_data_dir': None,  # e.g., '/Volumes/data_network/data/mms'
          'debug_mode': False,
          'download_only': False,
          'no_download': False}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MIT License
#
# Copyright (c) 2020 Louis Richard
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so.

import numpy as np

from astropy import constants


def dynamic_press(n, v, s="i"):
    """Computes dynamic pressure.

    Parameters
    ----------
    n : xarray.DataArray
        Time series of the number density of the specie.

    v : xarray.DataArray
        Time series of the bulk velocity of the specie.

    s : str {"i", "e"}, optional
        Specie. default "i".

    Returns
    -------
    p_dyn : xarray.DataArray
        Time series of the dynamic pressure of the specie.

    Examples
    --------
    >>> from pyrfu import mms, pyrf

    Time interval

    >>> tint = ["2019-09-14T07:54:00.000", "2019-09-14T08:11:00.000"]

    Spacecraft index

    >>> mms_id = 1

    Load ion bulk velocity and remove spintone

    >>> v_xyz_i = mms.get_data("Vi_gse_fpi_fast_l2", tint, mms_id)
    >>> st_xyz_i = mms.get_data("STi_gse_fpi_fast_l2", tint, mms_id)
    >>> v_xyz_i = v_xyz_i - st_xyz_i

    Ion number density

    >>> n_i = mms.get_data("Ni_fpi_fast_l2", tint, mms_id)

    Compute dynamic pressure

    >>> p = pyrf.dynamic_press(n_i, v_xyz_i, s="i")

    """

    if s == "i":
        m = constants.m_p.value
    elif s == "e":
        m = constants.m_e.value
    else:
        raise ValueError("Unknown specie")

    p_dyn = n * m * np.linalg.norm(v, axis=0) ** 2

    return p_dyn



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MIT License
#
# Copyright (c) 2020 Louis Richard
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so.

from astropy.time import Time


def iso2unix(t):
    """
    Converts time in iso format to unix

    Parameters
    ----------
    t : str or list of str
        Time.

    Returns
    -------
    out : float or list of float
        Time in unix format.

    """

    # Convert iso time to unix
    out = Time(t, format="iso").unix

    return out



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MIT License
#
# Copyright (c) 2020 Louis Richard
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so.

import xarray as xr


def ts_scalar(t, data, attrs=None):
    """Create a time series containing a 0th order tensor

    Parameters
    ----------
    t : ndarray
        Array of times.

    data : ndarray
        Data corresponding to the time list.

    attrs : dict
        Attributes of the data list.

    Returns
    -------
    out : xarray.DataArray
        0th order tensor time series.

    """

    if data.ndim != 1:
        raise TypeError("Input must be a scalar")

    if len(t) != len(data):
        raise IndexError("Time and data must have the same length")

    flag_attrs = True

    if attrs is None:
        flag_attrs = False

    out = xr.DataArray(data, coords=[t[:]], dims="time")

    if flag_attrs:
        out.attrs = attrs

    out.attrs["TENSOR_ORDER"] = 0

    return out
